% ex_lm_test_kf.m
% manobras com o mdulo lunar observadas por um filtro de Kalman.

% Especificaes do mdulo
m = 4800;

% Especificaes da Lua
g = 1.625;

% Especificaes dos Propulsores
Ehmax = 45000;
Edmax = 45000;
Eo = m*g;
Eopc = 100*Eo/Ehmax;
W = 450*eye(2);

% Especificaes do rastreamento
V = 15*eye(2);

% Modelo contnuo do mdulo
A = [0 1 0 0; 0 0 0 0; 0 0 0 1; 0 0 0 0];
B = [0 0; 1/m 0; 0 0; 0 1/m];
C = [1 0 0 0; 0 0 1 0];
D = [0 0; 0 0];
Sc  = ss(A,B,C,D);
Scx = ss(A,B,eye(4),zeros(4,2));

% Modelo discretizado
T = 0.1;
Sd  = c2d(Sc,T,'zoh');
Sdx = c2d(Scx,T,'zoh');

% Filtro de Kalman

% Sistema aumentado para projetar o filtro (Veja '>> doc kalman' no Matlab)
Phi = Sd.a;
Gamma = Sd.b;
Lambda = Gamma;
H = zeros(2,2);
Sdaug = ss(Phi,[Gamma Lambda],C,[D H],T); 

[KF,L,P]=kalman(Sdaug,eye(2),W,V); 

% Simulao da manobra

xo = [1000 0 -1000 0];

tEh =  [0   5  25  35  40];
Eh  = ([0 -20  20   0  20] + Eopc)/100*Ehmax;
Eh_data = [tEh' Eh'];

tEd = [0   10  20  30 40];
Ed  = [60 -60   0 -60 60]/100*Edmax;
Ed_data = [tEd' Ed'];


sim('ex_lm_test_kf_sim')



figure(1);
plot(ysim(:,2),ysim(:,1),'r.','markersize',8)
hold on
plot(xsim(:,3),xsim(:,1),'b','linewidth',2)
plot(ysim_clean(:,2),ysim_clean(:,1),'b--','linewidth',1)
hold off
axis([-1100 100 -100 1100])
grid on
grid on
set(gca,'fontname','times','fontsize',16)
xlabel('distncia  base d (m)');
ylabel('altitude h (m)')
legend('(h,d) medido','(h,d) efetivo (no medido)','(h,d) sem rudo (fictcio)')

figure(2);
h = plot(tsim,xkf(:,1)-xsim(:,1),tsim,xkf(:,2)-xsim(:,2),tsim,xkf(:,3)-xsim(:,3),tsim,xkf(:,4)-xsim(:,4));
set(h(1),'linewidth',2,'color',[0 0 1])
set(h(2),'linewidth',1,'color',[0 0 0.6])
set(h(3),'linewidth',2,'color',[1 0 0])
set(h(4),'linewidth',1,'color',[0.6 0 0])
grid on
set(gca,'fontname','times','fontsize',12)
xlabel('t (s)');
ylabel('erro de estimao (entre o estado estimado e o estado efetivo)')
legend({'erro em x_{1} = y_{1}','erro em x_{2}','erro em x_{3} = y_{2}','erro em x_{4}'},'location','southwest')

figure(3);
h = plot(tsim,xkf(:,1)-xsim_clean(:,1),tsim,xkf(:,2)-xsim_clean(:,2),tsim,xkf(:,3)-xsim_clean(:,3),tsim,xkf(:,4)-xsim_clean(:,4));
set(h(1),'linewidth',2,'color',[0 0 1])
set(h(2),'linewidth',1,'color',[0 0 0.6])
set(h(3),'linewidth',2,'color',[1 0 0])
set(h(4),'linewidth',1,'color',[0.6 0 0])
grid on
set(gca,'fontname','times','fontsize',12)
xlabel('t (s)');
ylabel('erro de estimao (entre o estado estimado e o estado sem rudo)')
legend({'erro em x_{1} = y_{1}','erro em x_{2}','erro em x_{3} = y_{2}','erro em x_{4}'},'location','northwest')
